<?php

namespace Teknisa\FIN\Modules\ContaPagar\Factory;

use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Baixapaga;
use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Teknisa\Model\Financeiro\Entities\Errcnabbolp;
use Teknisa\Model\Financeiro\Entities\Emprbanc;
use Teknisa\Model\Financeiro\Entities\GeralPag;
use Teknisa\Model\Financeiro\Entities\Ajbolpag;
use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;

class RetornoPagarFactory {

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    public function insereCnabPag($nrorg, $cdempresa, $nrcnabpag, $idtipcnabpag, $cdoperador, $cdbanco, $dsArquivo, $idprenegociado = 'N'){

        $cnabpag = new Cnabpag();
        $cnabpag->setNrorg($nrorg);
        $cnabpag->setCdempresa($cdempresa);
        $cnabpag->setNrcnabpag($nrcnabpag);
        $cnabpag->setDsarqcnabpag($dsArquivo);
        $cnabpag->setIdtipcnabpag($idtipcnabpag);
        $cnabpag->setDtcnabpag(DateUtil::getDataAtual());
        $cnabpag->setCdoperador($cdoperador);
        $cnabpag->setCdbanco($cdbanco);
        $cnabpag->setDtinclusao(DateUtil::getDataAtual());
        $cnabpag->setIdprenegociado($idprenegociado);
        return $cnabpag;
    }

    public function getEmprbanc($cdbanco, $cdempresa, $nrorg){
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdbanco'] = $cdbanco;
        $params['cdempresa'] = $cdempresa;
        $emprbanc = $this->entityManager->getRepository(Emprbanc::class)->findOneBy($params);
        return $emprbanc;
    }

    public function getBoletoPagar($cdfilial, $nrlanbolpag, $nrorg){

        $params = array();
        $params['nrorg'] = $nrorg;
        $params['nrlanbolpag'] = $nrlanbolpag;
        $params['cdfilial'] = $cdfilial;
        $boletpagar = $this->entityManager->getRepository(Boletpagar::class)->findOneBy($params);
        return $boletpagar;
    }

    public function alteraBoleto(Boletpagar $boletpagar, $vrfinalbpag, $cdfilibaipag,
                                        $nrbaixapag, $nrcnabpagret, $idsitucnabp, $statusBoleto){
            $boletpagar->setVrfinalbpag($vrfinalbpag);
            $boletpagar->setCdfilibaipag($cdfilibaipag);
            $boletpagar->setNrbaixapag($nrbaixapag);
            $boletpagar->setNrcnabpagret($nrcnabpagret);
            $boletpagar->setIdsitucnabp($idsitucnabp);
            $boletpagar->setIdstatbpag($statusBoleto);
            return $boletpagar;
    }

    public function alteraSituacaoCnabPag(Boletpagar $boletpagar, $nrcnabpagret, $idsitucnabp){
        $boletpagar->setNrcnabpagret($nrcnabpagret);
        $boletpagar->setIdsitucnabp($idsitucnabp);
        return $boletpagar;
    }

    public function insereBaixa($cdempresa, $cdfilial, $cdtpbaixapag, $dtpagamento,
                                $cdreducont, $cdoperador, $nrorg, $nrbaixapag, $vrbaixa,
                                $nrcnabpag, $idstatuspag){

        $baixapaga = new Baixapaga();
        $baixapaga->setNrbaixapag($nrbaixapag);
        $baixapaga->setVrbaixapag($vrbaixa);
        $baixapaga->setCdempresa($cdempresa);
        $baixapaga->setCdfilial($cdfilial);
        $baixapaga->setIdstatusbpag($idstatuspag);
        $baixapaga->setCdtpbaixapag($cdtpbaixapag);
        $baixapaga->setDtbaixapag($dtpagamento);
        $baixapaga->setCdreducont($cdreducont);
        $baixapaga->setCdoperador($cdoperador);
        $baixapaga->setDtlancbaixap(DateUtil::getDataAtual());
        $baixapaga->setNrorg($nrorg);
        $baixapaga->setDsbaixapag("Baixa realizada via importação CNAB. Retorno nº {$nrcnabpag}");
        $baixapaga->setIdstemicheq('N');
        $baixapaga->setDtinclusao(DateUtil::getDataAtual());
        $baixapaga->setDtultatu(DateUtil::getDataAtual());
        $baixapaga->setCdoperinclusao($cdoperador);
        $baixapaga->setCdoperultatu($cdoperador);
        $baixapaga->setIdativo('S');
        return $baixapaga;
    }

    public function baixaBoleto($cdfilial, $nrlanbolpag, $nrorg, $nrbaixapag){

        $params = array();
        $params['nrorg'] = $nrorg;
        $params['nrlanbolpag'] = $nrlanbolpag;
        $params['cdfilial'] = $cdfilial;
        $boletpagar = $this->entityManager->getRepository(Boletpagar::class)->findOneBy($params);
        return $boletpagar;
    }

    public function getBaixaPaga($cdfilial, $nrbaixapag, $nrorg){

        $params = array();
        $params['nrorg'] = $nrorg;
        $params['nrbaixapag'] = $nrbaixapag;
        $params['cdfilial'] = $cdfilial;
        $baixapaga = $this->entityManager->getRepository(Baixapaga::class)->findOneBy($params);
        return $baixapaga;
    }

    public function insereValorBaixa($baixapaga, $valor){

        $baixapaga->setVrbaixapag($baixapaga->getVrbaixapag() + $valor);
        return $baixapaga;
    }

    public function insereOcorencia($cdfilial, $nrlanbolpag, $cdbanco, $cdercnabbolp, $nrorg){

        $errcnabbolp = new Errcnabbolp();
        $errcnabbolp->setCdfilial($cdfilial);
        $errcnabbolp->setNrlanbolpag($nrlanbolpag);
        $errcnabbolp->setNrorg($nrorg);
        $errcnabbolp->setCdbanco($cdbanco);
        $errcnabbolp->setCdercnabbolp($cdercnabbolp);
        $errcnabbolp->setDtinclusao(DateUtil::getDataAtual());
        return $errcnabbolp;
    }

    /**
     * Busca Cnab Pagar
     * @param $cdempresa
     * @param $nrcnabpag
     * @param $nrorg
     * @return Cnabpag
     */
    public function getCnabpag($cdempresa, $nrcnabpag, $nrorg){
        /** @var $params */
        $params = array();
        $params['nrorg']     = $nrorg;
        $params['cdempresa'] = $cdempresa;
        $params['nrcnabpag'] = $nrcnabpag;

        /** @var Cnabpag $cnabpag */
        $cnabpag = $this->entityManager->getRepository(Cnabpag::class)->findOneBy($params);
        return $cnabpag;
    }

    /**
     * Busca Ocorrências do Título a Pagar
     * @param $nrlanbolpag
     * @param $cdfilial
     * @param $nrorg
     * @return Errcnabbolp
     */
    public function getErrosCnab($nrlanbolpag, $cdfilial, $nrorg){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolpag'] = $nrlanbolpag;

        /** @var Errcnabbolp $errcnabbolps */
        $errcnabbolps = $this->entityManager->getRepository(Errcnabbolp::class)->findBy($params);
        return $errcnabbolps;
    }

    /**
     * Busca Boleto Retorno
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $nrorg
     * @param $nrcnabpagret
     * @return Boletpagar
     */
    public function getBoletoRetorno($cdfilial, $nrlanbolpag, $nrorg, $nrcnabpagret){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanbolpag']  = $nrlanbolpag;
        $params['nrcnabpagret'] = $nrcnabpagret;

        /** @var Boletpagar $boletpagar */
        $boletpagar = $this->entityManager->getRepository(Boletpagar::class)->findOneBy($params);
        return $boletpagar;
    }

    /**
     * Busca Geral Pagar
     * @param $nrorg
     * @return GeralPag
     */
    public function getGeralPag($nrorg){
        /** @var  $params */
        $params = array();
        $params['nrorg']      = $nrorg;

        /** @var GeralPag $geralpag */
        $geralpag = $this->entityManager->getRepository(GeralPag::class)->findOneBy($params);

        return $geralpag;
    }

    /**
     * Insere Ajuste Boleto
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $cdajuctpag
     * @param $vrajbolpag
     * @return Ajbolpag
     */
    public function inserirAjusteBoleto($nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag, $vrajbolpag){
        /** @var Ajbolpag $ajuste */
        $ajuste = new Ajbolpag();
        $ajuste->setNrorg($nrorg);
        $ajuste->setCdfilial($cdfilial);
        $ajuste->setNrlanbolpag($nrlanbolpag);
        $ajuste->setCdajuctpag($cdajuctpag);
        $ajuste->setVrajbolpag($vrajbolpag);
        $ajuste->setDtinclusao(DateUtil::getDataAtual());
        return $ajuste;
    }
}